//ParticleSystem.cpp

#include "ParticleSystem.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <GL/glut.h>


ParticleSystem test;
GLfloat tx, ty, tz;

/* Zuflligkeit fr Wind, damit das Tuch realistischer aussieht */
int		WindRandom (int min, int max);


/** Globale Variablen
*/

float	WindForceFactor = WINDFACTOR;




/** Konstruktor des Partikel-Systems bzw. Meshs
*/

ParticleSystem::ParticleSystem (int b, int l, int rb, int rl, int c, float h)
{
	iWidth = b;
	iLength	= l;
	iResWidth = rb;
	iResLength = rl;
	iSpringCount = c;
	fHeight	= h;
	
	mMesh = new Particle*[iResWidth];			//Anzahl der Particle
	for (int i=0; i<iResWidth; i++)
	{
		mMesh[i] = new Particle[iResLength];
	}


	GenerateMesh();								//Generierung des Meshs

	
	mSprings = new Spring[iSpringCount];		//Anzahl der Springs
	
	/* Schleife ber alle Particle */
	int springIndex = 0;
	for ( b=0; b<iResWidth; b++ )
	{
		for ( l=0; l<iResLength; l++ )
		{

			/* Initialisierung der Springs mit Adresse(&) der Particles */

			/* horizontale Federn (parallel zur x-Achse) */
			if (b < iResWidth-1)
			{
			mSprings[springIndex] = Spring( &mMesh[b][l], &mMesh[b+1][l  ] , SPRINGTENSIONCONSTANT, SPRINGDAMPINGCONSTANT );
			springIndex++;
			}

			/* vertikale Federn (parallel zur z-Achse) */
			if (l < iResLength-1)
			{
			mSprings[springIndex] = Spring( &mMesh[b][l], &mMesh[b  ][l+1] , SPRINGTENSIONCONSTANT, SPRINGDAMPINGCONSTANT );
			springIndex++;
			}

			/* diagonale Federn (von rechts unten nach links oben) */
			if ((b < iResWidth-1) && (l < iResLength-1))
			{
			mSprings[springIndex] = Spring( &mMesh[b][l], &mMesh[b+1][l+1] , SPRINGSHEARCONSTANT, SPRINGDAMPINGCONSTANT );
			springIndex++;
			}

			/* diagonale Federn (von rechts oben nach links unten) */
			if ((b < iResWidth-1) && (l > 0))
			{
			mSprings[springIndex] = Spring( &mMesh[b][l], &mMesh[b+1][l-1] , SPRINGSHEARCONSTANT, SPRINGDAMPINGCONSTANT );
			springIndex++;
			}

		}
	}


}




/** Default-Konstruktor 
*/

ParticleSystem::ParticleSystem()
{

}




/** generiert initiales Mesh fr die Oberflche
*/

void ParticleSystem::GenerateMesh()
{
	
	float f;

	for ( int b=0; b<iResWidth; b++ )
	{
		for ( int l=0; l<iResLength; l++ )
		{
			

			/* berechnet die Masse dieses Partikels */
			
			if ((b == 0) && (l == 0))
				f = 1;
			else if ((b == iResWidth-1) && (l == 0))
				f = 2;
			else if ((b == 0) && (l == iResLength-1))
				f = 2;
			else if ((b == iResWidth-1) && (l == iResLength-1))
				f = 1;
			else if (((b == 0) || (b == iResWidth-1)) && ((l != 0) && (l != iResLength-1)))
				f = 3;
			else 
				f = 6;

			/* Masse pro Flche setzen, abhngig von der Art (Masse) der Kleidung */
			
			mMesh[b][l].setMass ((f * MASSPERFACE) / 3);

			mMesh[b][l].setInvMass ();


			/* setzt initiale Position des Particles */
			
			mMesh[b][l].setPosition (Vector (LSTEP*l, fHeight, BSTEP*b));

			
			/* setzt alle weiteren initialen Werte der Parameter von Particle */
			
			mMesh[b][l].setVelocity (Vector (0, 0, 0));

			mMesh[b][l].setAcceleration (Vector (0, 0, 0));

			mMesh[b][l].setDisplacement (Vector (0, 0, 0));

			mMesh[b][l].setForces (Vector (0, 0, 0));

			mMesh[b][l].setDrag (Vector (0, 0, 0));

			mMesh[b][l].setWind (Vector (WindRandom(0, 10), WindRandom(0, 5), 0));


			/** letzter Punkt fixiert;
				fr fallendes Tuch 
			*/

/*			if ((b==iResWidth-1) && (l==iResLength-1))
				mMesh[b][l].bFixed = true;
			else 
				mMesh[b][l].bFixed = false;
*/

			/** Tischdecke;
				quadratische Flche in der Mitte des Tuchs fixiert
			*/
			if ((b >= IRESWIDTH/6) && (b <= IRESWIDTH*5/6) && (l >= IRESLENGTH/6) && (l <= IRESLENGTH*5/6))
			{
				mMesh[b][l].bFixed = true;
			}
			else 
				mMesh[b][l].bFixed = false;


			/** Fahne (waagrecht);
				eine Seite des Tuchs fixiert
			*/
/*			if ((l == 0))
			{
				mMesh[b][l].bFixed = true;
			}
			else 
				mMesh[b][l].bFixed = false;
*/

 
		}
	}

}




/** aktualisiert das Mesh, wenn es bewegt wurde;
	hier werden alle Krfte berechnet;
*/

void ParticleSystem::UpdateMesh()
{
	
	for ( int b=0; b<iResWidth; b++ )
	{
		for ( int l=0; l<iResLength; l++ )
		{


			/** Schwerkraft
			*/

			mMesh[b][l].setForces ( Vector ( 0, 
											 (float) (GRAVITY * (mMesh[b][l].getMass())), 
											 0 ));

					

			/** Reibungsluftwiderstand
			*/

			Vector Drag = (mMesh[b][l].getVelocity());
			Drag.Reverse();
			Drag.Normalize();
			mMesh[b][l].setDrag (Drag);
			mMesh[b][l].addForces ( mMesh[b][l].getForces() 
									+ mMesh[b][l].getDrag() 
									* ( mMesh[b][l].getVelocity().computeLength() 
									*   mMesh[b][l].getVelocity().computeLength() ) 
									* DRAGCOEFFICIENT );
	


			/** Wind
			*/

/*			Vector Wind = (mMesh[b][l].getWind());
			Wind.Normalize();
			mMesh[b][l].addForces ( mMesh[b][l].getForces() 
									+ mMesh[b][l].getWind() 
									* WindRandom(0, WindForceFactor) ); 
*/													


			
			
		}
	}

	for ( int i=0; i<iSpringCount; i++ )
	{
		Particle* first = mSprings[i].getBeginParticle();
		Particle* second = mSprings[i].getEndParticle();


		/** Feder-Dmpfungs-Kraft
		*/

		Vector f1, f2;		// Kraft von first und gegengleiche Kraft von second

		first->setDisplacement ( Vector ( first->getPosition() - second->getPosition() ) );
		Vector Velo = first->getVelocity() - second->getVelocity();
		  
		
		float Springforce	= ((first->getDisplacement().computeLength() 
								- mSprings[i].getLength()) 
								* mSprings[i].getSpringConst());

		float Dampingforce	= (((Velo * first->getDisplacement()) 
								/ first->getDisplacement().computeLength()) 
								* mSprings[i].getDamping());

		float SpringDamping = -(Springforce + Dampingforce);

		f1 =  ( first->getDisplacement() / first->getDisplacement().computeLength() ) 
				* SpringDamping ;

		f2 = f1;
		f2.Reverse();

		if (first->bFixed == false)
			first->addForces ( f1 );
				
		if (second->bFixed == false)
			second->addForces ( f2 );

					
	}

}




/** Simulationsschritt dt
*/


void ParticleSystem::StepSimulation (float dt) 
{

	float dtime = dt;

	/* alle Krfte berechnen */
	UpdateMesh();

	/* Integration */
	for ( int b=0; b<iResWidth; b++ )
	{
		for ( int l=0; l<iResLength; l++ )
		{
			if ( mMesh[b][l].bFixed == false )
			{


				mMesh[b][l].setAcceleration ( Vector ( mMesh[b][l].getForces() / mMesh[b][l].getMass() ) );

				mMesh[b][l].setVelocity ( Vector ( mMesh[b][l].getVelocity() + mMesh[b][l].getAcceleration() * dtime ) );

				mMesh[b][l].setPosition ( Vector ( mMesh[b][l].getPosition() + mMesh[b][l].getVelocity() * dtime ) );
		
			}
		}
	}
}




/** Berechnung des Zufallswertes fr den Wind
*/

int WindRandom (int min, int max) 
{
	int RandomNumber;

	RandomNumber = (((abs(rand())%(max-min+1))+min));

	if (RandomNumber>max) 
	{
		RandomNumber = max;
	}

	if (RandomNumber<min) 
	{
		RandomNumber = min;
	}

	return RandomNumber;
}




/** Windfaktor
*/

void ParticleSystem::SetWindForceFactor(float wf)
{
	WindForceFactor = WINDFACTOR;
}




/** Display-Funktion
	
	hier wird das Mesh mithilfe von OpenGL
	im Uhrzeigersinn gezeichnet

*/

void display(void)
{

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	//bereinigt Window von alter Farbe
	glColor3f (1.0, 1.0, 1.0);								//Farbe fr zu zeichnende Objekte

	glLoadIdentity();										//Einheitsmatrix
	
	gluLookAt(tx, ty, tz, test.iWidth/2, test.fHeight, test.iLength/2, 0.0, 1.0, 0.0);	//View Transformation

	
	for ( int b=0; b<(test.iResWidth-1); b++ )
	{
		for ( int l=0; l<(test.iResLength-1); l++ )
		{

			glColor3f( l / (float) test.iResLength, b / (float) test.iResWidth, 0.5 );	//andere Farbe fr Objekt
			
			glBegin(GL_POLYGON);	//definiert, dass das Objekt GL_POLYGON gezeichnet werden soll
			
			glVertex3f(test.mMesh[b  ][l  ].getPosition().getX(), test.mMesh[b  ][l  ].getPosition().getY(), test.mMesh[b  ][l  ].getPosition().getZ());

			glVertex3f(test.mMesh[b+1][l  ].getPosition().getX(), test.mMesh[b+1][l  ].getPosition().getY(), test.mMesh[b+1][l  ].getPosition().getZ());

			glVertex3f(test.mMesh[b+1][l+1].getPosition().getX(), test.mMesh[b+1][l+1].getPosition().getY(), test.mMesh[b+1][l+1].getPosition().getZ());

			glVertex3f(test.mMesh[b  ][l+1].getPosition().getX(), test.mMesh[b  ][l+1].getPosition().getY(), test.mMesh[b  ][l+1].getPosition().getZ());

			glEnd();				//definiert, dass Objekt fertig gezeichnet ist
		}
	}
	

	glutSwapBuffers ();
}



void init (void) 
{
	glClearColor (0.8, 0.8, 0.9, 0.0);						//neue Hintergrundfarbe des Fensters

   	glEnable(GL_DEPTH_TEST);								//Einschalten des Z-Buffers

	glViewport(0,0,500,500);								//Viewport (x, y, b, h)
	glMatrixMode(GL_PROJECTION);							//Projektionsmatrix
	glLoadIdentity();										//Einheitsmatrix

	gluPerspective(60.0f,(GLfloat)500/(GLfloat)500 ,0.1f,1000.0f);	//Perspektivische Projektion

	glMatrixMode(GL_MODELVIEW);								//Modelviewmatrix
	glLoadIdentity();										//Einheitsmatrix
}



/** Keyboard-Kommandos fr Partikel und Kamera:
	w = Wireframe ein
	W = Wireframe aus

	Ein Partikel bewegbar mit:
	h = positive y-Achse, jeweils um +0.1
	H = negative y-Achse, jeweils um -0.1
	u = positive x-Achse, jeweils um +0.1
	U = negative x-Achse, jeweils um -0.1
	n = positive z-Achse, jeweils um +0.1
	N = negative z-Achse, jeweils um -0.1

	s = Auslsen der Simulation (Taste gedrckt halten)

	Kamera bewegbar mit:
	x = positive x-Achse
	X = negative x-Achse
	y = positive y-Achse
	Y = negative y-Achse
	z = positive z-Achse
	Z = negative z-Achse
*/

void keyboard(unsigned char key, int x, int y)
{
	switch (key) 
	{
		case 'w':
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			glutPostRedisplay();
			break;
		case 'W':
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			glutPostRedisplay();
			break;

		case 'h':
			test.mMesh[0][0].setDisplacement ( Vector (0, 0.1, 0));
			test.mMesh[0][0].setPosition ( test.mMesh[0][0].getPosition() + Vector (0, 0.1, 0));
			test.StepSimulation(0.010);
			glutPostRedisplay();
			break;
		case 'H':
			test.mMesh[0][0].setDisplacement ( Vector (0, -0.1, 0));
			test.mMesh[0][0].setPosition ( test.mMesh[0][0].getPosition() + Vector (0, -0.1, 0));
			test.StepSimulation(0.010);
			glutPostRedisplay();
			break;

		case 'u':
			test.mMesh[0][0].setDisplacement ( Vector (0.1, 0, 0));
			test.mMesh[0][0].setPosition ( test.mMesh[0][0].getPosition() + Vector (0.1, 0, 0));
			test.StepSimulation(0.010);
			glutPostRedisplay();
			break;
		case 'U':
			test.mMesh[0][0].setDisplacement ( Vector (-0.1, 0, 0));
			test.mMesh[0][0].setPosition ( test.mMesh[0][0].getPosition() + Vector (-0.1, 0, 0));
			test.StepSimulation(0.010);
			glutPostRedisplay();
			break;

		case 'n':
			test.mMesh[0][0].setDisplacement ( Vector (0, 0, 0.1));
			test.mMesh[0][0].setPosition ( test.mMesh[0][0].getPosition() + Vector (0, 0, 0.1));
			test.StepSimulation(0.010);
			glutPostRedisplay();
			break;
		case 'N':
			test.mMesh[0][0].setDisplacement ( Vector (0, 0, -0.1));
			test.mMesh[0][0].setPosition ( test.mMesh[0][0].getPosition() + Vector (0, 0, -0.1));
			test.StepSimulation(0.010);
			glutPostRedisplay();
			break;
	
		case 's':
			test.StepSimulation(0.016);
			glutPostRedisplay();
			break;

		case 'x':
			tx += 0.1;
			glutPostRedisplay();
			break;
		case 'X':
			tx -= 0.1;
			glutPostRedisplay();
			break;

		case 'y':
			ty += 0.1;
			glutPostRedisplay();
			break;
		case 'Y':
			ty -= 0.1;
			glutPostRedisplay();
			break;

		case 'z':
			tz += 0.1;
			glutPostRedisplay();
			break;
		case 'Z':
			tz -= 0.1;
			glutPostRedisplay();
			break;

		default:
         break;
   }
}



int main(int argc, char** argv)
{
	test = ParticleSystem( IWIDTH, ILENGTH, IRESWIDTH, IRESLENGTH, NUMSPRINGS, HEIGHT);

	/* Position der Kamera */
	tx = -5;
	ty = 2;
	tz = -5;

	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize (600, 600); 
	glutInitWindowPosition (50, 50);
	glutCreateWindow ("Christiane's Particleparty");
	init ();
	glutDisplayFunc(display); 
	glutKeyboardFunc(keyboard);
	glutMainLoop();
	return 0;   /* ANSI C requires main to return int. */
}
