//ParticleSystem.h

#ifndef PARTICLESYSTEM_H
#define PARTICLESYSTEM_H
#include "Particle.h"
#include "Spring.h"
#include <iostream>

class ParticleSystem
{
	public:
		int		iWidth, iLength;			// Breite, Lnge des Meshs
		int		iResWidth, iResLength;		// Auflsung Breite, Lnge des Meshs
		int		iSpringCount;				// Anzahl der Federn
		float	fHeight;					// Hhe des gesamten Meshs
		
		Particle** mMesh;					// 2D-Array von Particles
		Spring* mSprings;					// 1D-Array von Springs
		
		void GenerateMesh();				// Generierung des Meshs; Setzen aller Krfte auf Null
		void UpdateMesh();					// Aktualisierung des Meshs; Berechnung aller Krfte
		void StepSimulation (float dt);		// Zeitschritt dt in der Simulation
		void display();						// Display-Funktion: Zeichnen des Meshs
		void SetWindForceFactor (float wf);	// Windstrke setzen


		/** Konstruktor und Default-Konstruktor des Partikel-Systems
		*/
		ParticleSystem (int b, int l, int rb, int rl, int c, float h);
		ParticleSystem ();					

};

/** verschiedene Konstanten
*/
#define		GRAVITY					-9.81	// Einheit: m/s^2
#define		SPRINGTENSIONCONSTANT	400.0f	// Federkonstante vertikal & horizontal
#define		SPRINGSHEARCONSTANT		600.0f	// Federkonstante diagonal
#define		SPRINGDAMPINGCONSTANT	2.0f	// Dmpfungskonstante
#define		DRAGCOEFFICIENT			0.2f	// Luftwiderstandskoeffizient
#define		WINDFACTOR				4		// Windfaktor
#define		CLOTHMASS				35.0f	// Gesamtmasse der Kleidung
#define		MASSPERFACE				(CLOTHMASS/ (float) (((IWIDTH) * (ILENGTH)) *2))	// Masse pro Flche

/** Werte fr den Konstruktor
*/
#define		IWIDTH					7
#define		ILENGTH					7
#define		IRESWIDTH				70
#define		IRESLENGTH				70
#define		NUMSPRINGS				(((IRESWIDTH-1) * IRESLENGTH) + (IRESWIDTH * (IRESLENGTH-1)) + ((IRESWIDTH-1) * (IRESWIDTH-1)) + ((IRESLENGTH-1) * (IRESLENGTH-1)))	//Anzahl der Federn
#define		HEIGHT					-5

/** Werte fr Breite und Lnge
*/
#define		BSTEP					((float) IWIDTH / (float)(IRESWIDTH-1))		// Gre des Zwischenraums zwischen 2 Partikeln (Breite)
#define		LSTEP					((float) ILENGTH / (float)(IRESLENGTH-1))	// Gre des Zwischenraums zwischen 2 Partikeln (Lnge)


#endif