//Vector.h

#ifndef VECTOR_H
#define VECTOR_H
#include <math.h>

class Vector 
{
		float x;
		float y;
		float z;
		float tol;
	
	public:
		Vector ( float x, float y, float z );	// Konstruktor
		Vector ();								// Default-Konstruktor
		Vector ( const Vector& in );			// Copy-Konstruktor


		
		/** set-Methoden
		*/

		inline void setX (float _x) 
		{
			x = _x;
		}
		
		inline void setY (float _y) 
		{
			y = _y;
		}
		
		inline void setZ (float _z) 
		{
			z = _z;
		}

		
		/** get-Methoden
		*/

		inline float getX () const 
		{
			return x;
		}
		
		inline float getY () const 
		{
			return y;
		}
		
		inline float getZ () const 
		{
			return z;
		}

		
		/** Berechnung der Lnge des Vektors
		*/

		inline float computeLength () 
		{
			return (float) (sqrt (x * x + y * y + z * z));

		}

		
		/** Berechnung der Normalen des Vektors
		*/

		inline void Normalize () 
		{
			float n = (float) sqrt (x * x + y * y + z * z);
			if (n <= tol) n = 1;
			x /= n;
			y /= n;
			z /= n;

			if (fabs (x) < tol) x = 0.0f;
			if (fabs (y) < tol) y = 0.0f;
			if (fabs (z) < tol) z = 0.0f;
		}

		
		/** Umkehrung
		*/

		inline void Reverse () 
		{
			x = -x;
			y = -y;
			z = -z;
		}

		
		/** Addition
		*/

		inline Vector operator+(Vector i) 
		{
			return Vector(i.x + x, i.y + y, i.z + z);
		}


		/** Subtraktion
		*/

		inline Vector operator-(Vector i) 
		{
			return Vector(x - i.x, y - i.y, z - i.z);
		}

		
		/** Multiplikation
		*/
		
		inline Vector operator*(float s) 
		{
			return Vector(x*s, y*s, z*s);
		}

		
		/** Division
		*/	

		inline Vector operator/(float s) 
		{
			return Vector(x/s, y/s, z/s);
		}

		
		/** Scalar Multiplikation
		*/

		inline Vector& operator*=(float s) 
		{
			x *=s;
			y *=s;
			z *=s;
			return *this;
		}


		/** Scalar Division
		*/

		inline Vector& operator/=(float s) 
		{
			x /=s;
			y /=s;
			z /=s;
			return *this;
		}

		
		/** Zuweisung
		*/

		inline Vector operator=(Vector i) 
		{
 			return Vector(x = i.x, y = i.y, z = i.z);
		}

		
		/** Skalarprodukt
		*/

		inline float operator*(Vector i) 
		{
			return (x * i.x + y * i.y + z * i.z);
		}


};


#endif
